open Nstring

let ip_conntrack = "/proc/net/ip_conntrack" ;;

type proto_t =
    UDP
  | TCP
  | ICMP
;;

type assure_t =
    ASSURED
  | UNREPLIED
  | UNSPECIFIED
;;

type state_t =
    NONE
  | ESTABLISHED
  | SYN_SENT
  | SYN_RECV
  | FIN_WAIT
  | TIME_WAIT
  | CLOSE
  | CLOSE_WAIT
  | LAST_ACK
  | LISTEN
;;

type conn_t = {
  proto: proto_t;
  unknown1: int;
  unknown2: int;
  state: state_t;
  r_src: Unix.inet_addr;
  r_dst: Unix.inet_addr;
  r_sport: int;
  r_dport: int;
  t_src: Unix.inet_addr;
  t_dst: Unix.inet_addr;
  t_sport: int;
  t_dport: int;
  assure: assure_t;
  use: int;
}
;;

exception Entry_is_not_tcp ;;
exception Bad_conntrack_data of string ;;

let parse_line str = 
  let p_proto t = 
    match t with
	"tcp" -> TCP
      | _ -> raise Entry_is_not_tcp
  and p_state t =
    match t with
	"NONE" -> NONE
      | "ESTABLISHED" -> ESTABLISHED
      | "SYN_SENT" -> SYN_SENT
      | "SYN_RECV" -> SYN_RECV
      | "FIN_WAIT" -> FIN_WAIT
      | "TIME_WAIT" -> TIME_WAIT
      | "CLOSE" -> CLOSE
      | "CLOSE_WAIT" -> CLOSE_WAIT
      | "LAST_ACK" -> LAST_ACK
      | "LISTEN" -> LISTEN
      | _ -> raise (Bad_conntrack_data "Bad state in ip_conntrack.")
  in

  let o_proto = p_proto (trim (lgrab str ' '))
  and str = trim (rgrab str ' ') in

  let o_assure = (if (contains_substr str "[ASSURED]") then ASSURED
		  else if (contains_substr str "[UNREPLIED]") then UNREPLIED
		  else UNSPECIFIED) in

  let o_unknown1 = int_of_string (trim (lgrab str ' '))
  and str = trim (rgrab str ' ') in

  let o_unknown2 = int_of_string (trim (lgrab str ' '))
  and str = trim (rgrab str ' ') in

  let o_state = p_state (trim (lgrab str ' '))
  and str = trim (rgrab str '=') in

  let o_r_src = Unix.inet_addr_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  let o_r_dst = Unix.inet_addr_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  let o_r_sport = int_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  let o_r_dport = int_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  (* may be [UNREPLIED] or t_src: skip the [UNREPLIED] *)

  let o_t_src = Unix.inet_addr_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  let o_t_dst = Unix.inet_addr_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  let o_t_sport = int_of_string (lgrab str ' ')
  and str = trim (rgrab str '=') in

  let o_t_dport = int_of_string (lgrab str ' ')
  and str = trim (rgrab str ' ') in

  (* may be [ASSURED] or use=1, skip the [ASSURED] *)

  let o_use = int_of_string (trim (rgrab str '=')) in

    { proto=o_proto; unknown1=o_unknown1; unknown2=o_unknown2; state=o_state;
      r_src=o_r_src; r_dst=o_r_dst; r_sport=o_r_sport; r_dport=o_r_dport;
      t_src=o_t_src; t_dst=o_t_dst; t_sport=o_t_sport; t_dport=o_t_dport;
      assure=o_assure; use=o_use; }
;;

let read_conntrack =
  let input = ref ([] : conn_t list) in
  let fh = open_in ip_conntrack in
    try
      while true do
	try
	  input := (parse_line (input_line fh)) :: !input
	with
	    Entry_is_not_tcp -> ()
      done ;
      !input
    with
	End_of_file -> close_in fh ; !input
;;
