(* parsecfg - parses .ident's user config data
   
   (C) 2002 Nicholas Jay Kain <niklata@aerifal.cx>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation. 

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)

open Nstring

type hash_fields_t =
    UID
  | IP_ADDR
  | S_PORT
  | D_PORT
;;

type response_t =
    Deny
  | Accept
  | Spoof of string
  | Hash of hash_fields_t list
;;

type respond_t = {
  ip: Unix.inet_addr;
  mask: Unix.inet_addr;
(*  lport: int;
  hport: int; *)
  sports: int * int;
  dports: int * int;
  response: response_t;
}
;;

(* we will have to trap exceptions from this function *)
let parse_line str =
  try
  let buf = ref str
  and buft = ref ""
  and ip = ref ""
  and mask = ref ""
  and slow = ref "0"
  and shigh = ref "0"
  and dlow = ref "0"
  and dhigh = ref "0"
  and policy = ref Deny
  and hashes = ref ([]: hash_fields_t list)
  in
    buf := trim (String.lowercase !buf) ;
    ip := lgrab !buf ' ';
    buf := trim (rgrab !buf ' ') ;
    mask := lgrab !buf ' ';
    buf := trim (rgrab !buf ' ') ;

    buft := trim (lgrab !buf ' ') ;
    buf := trim (rgrab !buf ' ') ;

    if String.contains !buft ':' then
      begin
	slow := lgrab !buft ':' ;
	shigh := rgrab !buft ':'
      end
    else
      slow := !buft ;
    
    if String.contains !buf ':' then (* do we have a range or not? *)
      begin
	dlow := trim (lgrab !buf ':') ;
	buf := trim (rgrab !buf ':') ;
	dhigh := trim (lgrab !buf '-') ;
      end
    else
      dlow := trim (lgrab !buf '-') ;
    
    buf := trim (rgrab !buf '>') ;
    
    if String.sub !buf 0 5 = "spoof" then
      policy := Spoof (String.sub !buf 6 (String.length !buf - 6))
    else if String.sub !buf 0 6 = "accept" then
      policy := Accept
    else if String.sub !buf 0 4 = "hash" then
      begin
	buf := trim (String.sub !buf 4 (String.length !buf - 4)) ;
	while String.contains !buf ' ' do
	  begin
	    match lgrab !buf ' ' with
		"uid" -> hashes := UID :: !hashes
	      | "ip" -> hashes := IP_ADDR :: !hashes
	      | "sp" -> hashes := S_PORT :: !hashes
	      | "dp" -> hashes := D_PORT :: !hashes
	      | _ -> ()
	  end ;
	  buf := trim (rgrab !buf ' ')
	done ;
	policy := Hash !hashes
      end
    ; (* we implicitly assume Deny *)
    
    { ip = (Unix.inet_addr_of_string !ip);
      mask = (Unix.inet_addr_of_string !mask);
      sports = ((int_of_string !slow), (int_of_string !shigh));
      dports = ((int_of_string !dlow), (int_of_string !dhigh));
      response = (!policy); }
  with
      _ -> 
	{ ip = (Unix.inet_addr_of_string "0.0.0.0");
	  mask = (Unix.inet_addr_of_string "0.0.0.0");
	  sports = (0,0);
	  dports = (0,0);
	  response = Deny; }
;;

let read_file filename = 
  let policy = ref ([]: respond_t list) in
  let istream = open_in filename in
    try
      while true do
	policy := parse_line (input_line istream) :: !policy
      done ;
      !policy
    with
	End_of_file -> close_in istream ; !policy
;;
